"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const WavesConsoleAPI_1 = require("../../WavesConsoleAPI");
// TODO import Autocomplete from './Autocomplete';
const keycodes_1 = require("../lib/keycodes");
/**
 * @class {Input}
 * @extends {React.Component}
 */
class Input extends React.Component {
    /**
     * @constructor
     *
     * @param {IInputProps} props
     */
    constructor(props) {
        super(props);
        // history is set in the componentDidMount
        this.state = {
            value: props.value || '',
            multiline: false,
            rows: 1,
            historyCursor: props.history.length,
            hideSuggest: false
        };
        // Bind some methods to instance
        this.onChange = this.onChange.bind(this);
        this.onKeyPress = this.onKeyPress.bind(this);
    }
    /**
     * @method {onChange}
     *
     * @returns {undefined}
     */
    onChange() {
        if (!this.input) {
            return;
        }
        const { value } = this.input;
        const length = value.split('\n').length;
        this.setState({
            multiline: length > 1,
            rows: length < 20 ? length : 20,
            value,
        });
    }
    /**
     * @async
     * @method {onKeyPress}
     *
     * @param {React.KeyboardEvent} event
     *
     * @returns {Promise}
     */
    onKeyPress(event) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.input) {
                return;
            }
            const code = keycodes_1.default[event.keyCode];
            const { multiline } = this.state;
            const { history } = this.props;
            const command = this.input.value;
            // Clear console
            if (this.checkClearAction(event, code)) {
                return;
            }
            // Insert value from suggested commands
            if (this.checkShowSuggestAction(event, code)) {
                return;
            }
            // Insert closing bracket if needed
            this.checkAutoclosingAction(event);
            // Move in history if not in multiline mode
            if (!multiline && this.checkNotMultilineActions(event, code)) {
                return;
            }
            // Remove suggestions block if needed
            if (this.checkHideSuggestAction(event, code)) {
                return;
            }
            // Add command to history and try to execute it
            if (code === 'enter') {
                if (event.shiftKey) {
                    return;
                }
                if (!command) {
                    event.preventDefault();
                    return;
                }
                this.props.addHistory(command);
                this.setState({ historyCursor: history.length + 1, value: '', multiline: false });
                event.preventDefault();
                yield this.props.onRun(command);
                // Don't use `this.input.scrollIntoView();` as it messes with iframes
                window.scrollTo(0, document.body.scrollHeight);
                return;
            }
        });
    }
    /**
     * @method {checkClearAction}
     *
     * @param {React.KeyboardEvent} event
     * @param {string} code
     *
     * @returns {boolean}
     */
    checkClearAction(event, code) {
        // Clear console
        if (event.ctrlKey && code === 'l') {
            this.props.onClear();
            return true;
        }
        return false;
    }
    /**
     * @method {checkShowSuggestAction}
     *
     * @param {React.KeyboardEvent} event
     * @param {string} code
     */
    checkShowSuggestAction(event, code) {
        if (code == 'tab') {
            event.preventDefault();
            this.setCommandIntoInput();
            return true;
        }
        return false;
    }
    /**
     * @method {checkHideSuggestAction}
     *
     * @param {React.KeyboardEvent} event
     * @param {string} code
     *
     * @returns {boolean}
     */
    checkHideSuggestAction(event, code) {
        if (code === 'escape') {
            event.preventDefault();
            this.setState({ hideSuggest: true });
            return true;
        }
        else {
            this.setState({ hideSuggest: false });
        }
        return false;
    }
    /**
     * @method {checkAutoclosingAction}
     *
     * @param {React.KeyboardEvent} event
     */
    checkAutoclosingAction(event) {
        switch (event.key) {
            case '{':
            case '[':
            case '(':
            case '"':
            case "'":
                this.setClosingBracketOrQuoteIntoInput(event.key);
                break;
            case 'Backspace':
                this.unsetClosingBracketOrQuoteIntoInput();
                break;
        }
    }
    /**
     * @method {checkNotMultilineActions}
     *
     * @param {React.KeyboardEvent} event
     * @param {string} code
     *
     * @returns {boolean}
     */
    checkNotMultilineActions(event, code) {
        const { history } = this.props;
        let { historyCursor } = this.state;
        // Show back
        if (code === 'up arrow') {
            historyCursor--;
            if (historyCursor < 0) {
                this.setState({ historyCursor: 0 });
                return true;
            }
            this.setState({ historyCursor, value: history[historyCursor] });
            event.preventDefault();
            return true;
        }
        // Move forward
        if (code === 'down arrow') {
            historyCursor++;
            if (historyCursor >= history.length) {
                this.setState({ historyCursor: history.length, value: '' });
                return true;
            }
            this.setState({ historyCursor, value: history[historyCursor] });
            event.preventDefault();
            return true;
        }
        return false;
    }
    /**
     * Get entry for search from full input.value string
     *
     * @method {getCurrentCommandPiece}
     *
     * @returns {string}
     */
    getCurrentCommandPiece() {
        let { input } = this;
        let pos = input ? input.selectionStart : 0;
        let commands = (input ? input.value.substring(0, pos) : '').
            split(/[\s+()]/);
        // Get last entry from string
        if (commands && commands.length) {
            return commands.pop();
        }
        return '';
    }
    /**
     * Set closing bracket of given type
     *
     * @method {setClosingBracketOrQuoteIntoInput}
     *
     * @param {string} open
     */
    setClosingBracketOrQuoteIntoInput(open = '(') {
        // No need to go further
        if (!this.input) {
            return;
        }
        let brackets = Input.commasAndQuotes;
        let { input } = this;
        let pos = input.selectionStart || 0;
        let close = brackets[open] ? brackets[open] : brackets['('];
        // Set new value
        input.value = input.value.substring(0, pos) +
            close +
            input.value.substring(pos);
        // Set new caret position
        input.selectionStart = pos;
        input.selectionEnd = pos;
        // Re-render to cleanup
        this.setState({ value: input.value });
    }
    /**
     * Set closing bracket of given type
     *
     * @method {unsetClosingBracketOrQuoteIntoInput}
     */
    unsetClosingBracketOrQuoteIntoInput() {
        // No need to go further
        if (!this.input) {
            return;
        }
        let { input } = this;
        let pos = input.selectionStart || 0;
        let open = this.input.value.substr(pos - 1, 1);
        let close = Input.commasAndQuotes[open];
        // No need to go further
        if (!close) {
            return;
        }
        // Check if the closing symbol is similar to needed
        close = this.input.value.substr(pos, 1) === close ? close : '';
        // No need to go further
        if (!close) {
            return;
        }
        // Set new value
        input.value = input.value.substring(0, pos) +
            input.value.substring(pos + 1);
        // Set new caret position
        input.selectionStart = pos;
        input.selectionEnd = pos;
        // Re-render to cleanup
        this.setState({ value: input.value });
    }
    /**
     * Set command into input using found autocomplition variants
     *
     * @method {setCommandIntoInput}
     */
    setCommandIntoInput() {
        // No need to go further
        if (!this.input) {
            return;
        }
        let { input } = this;
        let beg = input.selectionStart || 0;
        let end = input.selectionEnd || 0;
        let pos = beg;
        let isFunc = false;
        let insert = this.getCurrentCommandPiece();
        let missing = '';
        let vocabulary = Input.commandsVocabulary;
        let command = '';
        let commands = this.getFilteredCommandsList();
        // Autocomplete works only if one value in list
        if (commands.length != 1) {
            return;
        }
        // Get full command
        command = commands[0];
        // No need to go further
        if (command === undefined || insert === undefined) {
            return;
        }
        // Check if it's method or member
        isFunc = vocabulary[command] && vocabulary[command].params !== undefined;
        // Get missing part of command name
        missing = command.substring(insert.length);
        // Set new value
        input.value = input.value.substring(0, beg) +
            missing + (isFunc ? '()' : '') +
            input.value.substring(end);
        // Set new caret position
        pos += missing.length;
        pos += isFunc ? 1 : 0;
        input.selectionStart = pos;
        input.selectionEnd = pos;
        // Re-render to cleanup
        this.setState({ value: input.value });
    }
    /**
     * Get list of commands filtered with needed command piece
     *
     * @method {getFilteredCommandsList}
     *
     * @returns {Array}
     */
    getFilteredCommandsList() {
        let seek = this.getCurrentCommandPiece();
        let list = [];
        if (seek) {
            // Get filtered list if possible
            list = Input.commandsList.filter((item) => {
                return item.indexOf(seek) === 0;
            });
            // Check for values inside
            if (list.length) {
                return list;
            }
        }
        return [];
    }
    /**
     * @method {render}
     *
     * @returns {React.Element}
     */
    render() {
        const rect = this.input ? this.input.getBoundingClientRect() : null;
        const dropdown = rect && rect.top < 50 ? true : false;
        const suggest = !this.state.hideSuggest ? this.createSuggest(dropdown) : null;
        const textarea = this.createTextarea();
        return (React.createElement("div", { className: "Input" },
            suggest,
            textarea));
    }
    /**
     * @method {createSuggest}
     *
     * @returns {React.Element}
     */
    createSuggest(dropdown) {
        const { value } = this.state;
        const { theme } = this.props;
        const commands = this.getFilteredCommandsList();
        return (React.createElement(SuggestRoot, { value: value, commands: commands, dropdown: dropdown, theme: theme }));
    }
    /**
     * @method {createTextarea}
     *
     * @returns {React.Element}
     */
    createTextarea() {
        const { autoFocus } = this.props;
        const { rows, value } = this.state;
        return (React.createElement("textarea", { className: "cli", rows: rows, autoFocus: autoFocus, ref: e => {
                this.input = e;
                this.props.inputRef(e);
            }, value: value, onChange: this.onChange, onKeyDown: this.onKeyPress }));
    }
}
/**
 * @static
 * @member {object} commandsVocabulary
 */
Input.commandsVocabulary = WavesConsoleAPI_1.WavesConsoleAPIHelp.texts;
/**
 * @static
 * @member {Array} commandsList
 */
Input.commandsList = Object.keys(WavesConsoleAPI_1.WavesConsoleAPIHelp.texts);
/**
 * @static
 * @member {object} commasAndQuotes
 */
Input.commasAndQuotes = {
    '(': ')',
    '{': '}',
    '[': ']',
    '"': '"',
    "'": "'"
};
exports.Input = Input;
/**
 * <SuggestRoot />
 *
 * @function {SuggestRoot}
 *
 * @param {ISuggestRootProps} props
 *
 * @returns {React.Element}
 */
function SuggestRoot(props) {
    const { value, theme, commands } = props;
    // No need to go further
    if (!commands || !commands.length || !value) {
        return null;
    }
    return (React.createElement("div", { className: 'Suggest' + (theme ? ' Suggest_theme_' + theme : '') },
        React.createElement(SuggestList, Object.assign({}, props)),
        "\u00A0"));
}
/**
 * <SuggestList />
 *
 * @function {SuggestList}
 *
 * @param {ISuggestRootProps} props
 *
 * @returns {React.Element}
 */
function SuggestList(props) {
    // No need to go further
    if (!props.commands || !props.commands.length) {
        return null;
    }
    const commands = props.commands.map((item, index) => {
        return (React.createElement(SuggestItem, { key: 'commands-suggest-item-' + index, title: item }));
    });
    return (React.createElement("ul", { className: 'Suggest__list' + (props.dropdown ? ' Suggest__list_drop_down' : '') }, commands));
}
/**
 * <SuggestItem />
 *
 * @function {SuggestItem}
 *
 * @param {object} props
 *
 * @returns {React.Element}
 */
function SuggestItem(props) {
    return (React.createElement("li", { className: 'Suggest__item' + (props.selected ? ' Suggest__item_is_selected' : '') }, props.title));
}
